/*
 * Copyright (C) 2020-2021 Suzhou Tiancheng Software Ltd.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/*
 * ls1x_gmac_hw.h
 *
 * Created on: 2013/07/01
 *     Author: Bian
 *
 */

#ifndef LS1X_GMAC_HW_H_
#define LS1X_GMAC_HW_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifndef bit
#define bit(x)	(1<<x)
#endif

/******************************************************************************
 * GMAC registers
 ******************************************************************************/

typedef struct LS1x_gmac_regs_s
{
	volatile unsigned int control;				/* 0xBFE10000 Configuration */
	volatile unsigned int framefilter;			/* 0xBFE10004 GMAC Frame Filter */
	volatile unsigned int hashhi;				/* 0xBFE10008 Hash Table High */
	volatile unsigned int hashlo;				/* 0xBFE1000C Hash Table Low */
	volatile unsigned int miictrl;				/* 0xBFE10010 GMII Address */
	volatile unsigned int miidata;				/* 0xBFE10014 GMII Data */
	volatile unsigned int flowctrl;				/* 0xBFE10018 Flow Control */
	volatile unsigned int vlantag;				/* 0xBFE1001C VLAN Tag */
	volatile unsigned int version;				/* 0xBFE10020 Version */
	volatile unsigned int rsv1[5];
	volatile unsigned int intstatus;			/* 0xBFE10038 Interrupt Status */
	volatile unsigned int intmask;				/* 0xBFE1003C Interrupt Mask */
	volatile unsigned int addr0hi;				/* 0xBFE10040 Address0 High */
	volatile unsigned int addr0lo;				/* 0xBFE10044 Address0 Low */
	volatile unsigned int addr1hi;				/* 0xBFE10048 Address1 High */
	volatile unsigned int addr1lo;				/* 0xBFE1004C Address1 Low */
	volatile unsigned int rsv2[28];
	volatile unsigned int anctrl;				/* 0xBFE100C0 AN Control */
	volatile unsigned int anstatus;				/* 0xBFE100C4 AN Status */
	volatile unsigned int anadvertise;			/* 0xBFE100C8 AN Advertisement */
	volatile unsigned int anlinkpa;				/* 0xBFE100CC AN Link Partner Ability */
	volatile unsigned int anexpansion;			/* 0xBFE100D0 AN Link Expansion */
	volatile unsigned int rsv3;
	volatile unsigned int miistatus;			/* 0xBFE100D8 SGMII/RGMII Status */

#if defined(LS1B)
	/*
	 * MMC registers of GMAC
	 */
	volatile unsigned int rsv4[12];
	volatile unsigned int mmc_imask_rx;         /* 0x010C mask for interrupts generated from rx counters */
	volatile unsigned int mmc_imask_tx;         /* 0x0110 mask for interrupts generated from tx counters */
	volatile unsigned int rsv5[60];
	volatile unsigned int mmc_imask_ipc;        /* 0x0200 mask for interrupts generated from rx IPC statistic counters */
#endif

} LS1x_gmac_regs_t;

/*
 * GMAC Config Register 0x0000
 */
typedef enum
{
	gmac_ctrl_tc			= bit(24),		/* Transmit Configuration in RGMII, ʹRGMII·Ϣ.
	      	  	  	  	  	  	  	  	  	   =1: ˫ģʽ, ·ٶ, ·Լ·/ϿϢͨRGMIIӿڴPHY */
	gmac_ctrl_wd			= bit(23),		/* Watchdog Disable, رտŹ.
	 	 	 	 	 	 	 	 	 	 	   =1: GMACرսն˵ĿŹʱ, Խ16384ֽڵ̫֡*/
	gmac_ctrl_jd			= bit(22),		/* Jabber Disable, رJabberʱ.
											   =1: GMACرշ͹еJabberʱ, Է16384ֽڵ̫֡ */
	gmac_ctrl_be			= bit(21),		/* Frame Burst Enable, =1: GMACʹܴе֡ͻģʽ */
	gmac_ctrl_je			= bit(20),		/* Jumbo Frame Enable - ֡ʹ, =1: GMACʹܾ֡(9018ֽ)Ľ */
	gmac_ctrl_ifg_mask		= (0x7<<17), 	/* Inter-Frame Gap - С֡ */
	gmac_ctrl_ifg_shift		= 17,
	gmac_ctrl_dcrs			= bit(16),		/* Disable Carrier Sense During Transmission, йرزͻ
											   =1: GMAC԰˫ģʽCRSźŵļ */
	gmac_ctrl_ps			= bit(15),		/* Port Select, 0: GMII (1000Mbps), 1: MII (10/100Mbps) */
	gmac_ctrl_fes			= bit(14),		/* Speed, 0: 10Mbps, 1: 100Mbps */
	gmac_ctrl_do			= bit(13),		/* Disable Receive Own, رսԼ̫֡.
											   =1: GMACհ˫ģʽgmii_txen_oЧ̫֡ */
	gmac_ctrl_lm			= bit(12),		/* Loopback Mode,  =1: GMII/MIIڻģʽ */
	gmac_ctrl_dm			= bit(11),		/* Duplex Mode, ʹȫ˫ģʽ
											   =1: GMACȫ˫ģʽ, ȫ˫ģʽ¿ͬʱͺͽ̫֡ */
	gmac_ctrl_ipc			= bit(10),		/* Checksum Offload, Ужʹ
											   =1: GMACӲյ̫֡ĸ(payload). IPV4ͷУǷȷ */
	gmac_ctrl_dr			= bit(9),		/* Disable Retry, رش
											   =1: GMACͻʱشͳͻ̫֡, ֻͻ */
	gmac_ctrl_lud			= bit(8),		/* Link Up/Down, 0: ·Ͽ, 1: ·  */
	gmac_ctrl_acs			= bit(7),		/* Automatic Pad/CRC Stripping, ̫֡Pad/CRCԶȥ
											   =1: GMACȥյ̫֡PadFCS */
	gmac_ctrl_bl_mask		= (0x03<<5), 	/* Back-Off Limit - , ƾslotӳʱ */
	gmac_ctrl_bl_shift		= 5,
	gmac_ctrl_bl_0			= (0<<5),		// 00: k=min(n,10)
	gmac_ctrl_bl_1			= (1<<5),		// 01: k=min(n,8)
	gmac_ctrl_bl_2			= (2<<5),		// 10: k=min(n,4)
	gmac_ctrl_bl_3			= (3<<5),		// 11: k=min(n,1)
	gmac_ctrl_dc			= bit(4),		/* Deferral Check, =1: ʹdeferral⹦ */
	gmac_ctrl_te			= bit(3),		/* Transmitter Enable, =1: ʹGMAC书 */
	gmac_ctrl_re			= bit(2),		/* Receiver Enable, =1: ʹGMACչ */
} LS1x_gmac_control_t;

/*
 * GMAC Frame Filter Register 0x0004
 */
typedef enum
{
	gmac_frmfilter_ra		= bit(31),		/* Receive All, =1: GMACģѽյ֡Ӧó, Դַ/Ŀַ˻ */
	gmac_frmfilter_hpf		= bit(10),		/* Hash or Perfect Filter, ϣȫ
											   =1: ڹϣ/ȫ˻ƥ̫֡͸Ӧ.
											   =0: ֻڹϣ˻ƥ̫֡ŷ͸Ӧ.  */
	gmac_frmfilter_saf		= bit(9),		/* Source Address Filter Enable, Դַʹ
											   GMAC COREȽϱȽϽյ̫֡ԴַSAĴеֵ, ƥ, ״̬ĴеSAMatchλΪ.
											   =1: Դַƥʧ, GMAC CORE̫֡.
											   =0: ԴַƥGMAC COREմ֡, ƥд״̬Ĵ. */
	gmac_frmfilter_saif		= bit(8),		/* SA Inverse Filtering, Դַת.
											   =1: SAĴԴַƥ̫֡ΪԴַƥʧ.
											   =0: SAĴԴַƥ̫֡ΪԴַƥʧ.  */
	gmac_frmfilter_pcf_mask	 = (0x03<<6),	/* bits: 7-6, Pass Control Frames, տ֡ */
	gmac_frmfilter_pcf_shift = 6,
	gmac_frmfilter_pcf_0	 = (0<<6),		// 00: GMACп֡;
	gmac_frmfilter_pcf_1	 = (1<<6),		// 01: GMACճpause֡п֡;
	gmac_frmfilter_pcf_2	 = (2<<6),		// 10: GMACп֡;
	gmac_frmfilter_pcf_3	 = (3<<6),		// 11: GMACݵַտ֡ */

	gmac_frmfilter_dbf		= bit(5),		/* Disable Broadcast Frames, رչ㲥֡. =1: нյĹ㲥֡. =0: й㲥֡.  */
	gmac_frmfilter_pm		= bit(4),		/* Pass All Multicast, жಥ֡. =1: жಥ֡. =0: жಥ֡.  */
	gmac_frmfilter_daif		= bit(3),		/* DA Inverse Filtering, Ŀַת.
											   =1: ԵͶಥ֡зĿַƥ.
											   =0: ԵͶಥ֡Ŀַƥ.  */
	gmac_frmfilter_hmc		= bit(2),		/* Hash Multicast, ϣಥ, =1: ԽյĶಥ֡ݹϣݽĿַ */
	gmac_frmfilter_huc		= bit(1),		/* Hash Unicast, ϣ; =1: Խյĵ֡ݹϣݽĿַ */
	gmac_frmfilter_pr		= bit(0),		/* Promiscuous Mode, ģʽ, =1: ̫֡ */
} LS1x_gmac_frmfilter_t;

/*
 * GMAC Flow Control Register 0x0018
 */
typedef enum
{
	gmac_flowctrl_pt_mask	= (0xFFFF<<16),		/* bits31~16, Pause Time, ͣʱ, 򱣴Ҫ봫֡еͣʱ */
	gmac_flowctrl_pt_shift  = 16,
	gmac_flowctrl_dzpq		= bit(7),			/* Disable Zero-Quanta Pause, ֹʱƬͣ֡, =1: ֹԶʱƬͣ֡Ĳ */
	gmac_flowctrl_plt_mask	= (0x03<<4),		/* bits5~4, Pause Low Threshold, ͣʱֵ */
	gmac_flowctrl_plt_shift = 4,
	gmac_flowctrl_plt_0		= (0<<4),			/* 00: ͣʱ4ʱ */
	gmac_flowctrl_plt_1		= (1<<4),			/* 01: ͣʱ28ʱ */
	gmac_flowctrl_plt_2		= (2<<4),			/* 10: ͣʱ144ʱ */
	gmac_flowctrl_plt_3		= (3<<4),			/* 11: ͣʱ256ʱ */
												/* һʱΪGMII/MIIӿϴ512ػ64ֽڵʱ */
	gmac_flowctrl_up		= bit(3),			/* Unicast Pause Frame Detect, ͣ֡̽,
												   =1: GMACGMACַ0ָıվַ̽ͣ֡ */
	gmac_flowctrl_rxfcen	= bit(2),			/* Receive Flow Control Enable, ʹ,
												   =1: GMACյͣ֡, Ұָͣ֡ʱͣ֡ķ */
	gmac_flowctrl_txfcen	= bit(1),			/* Transmit Flow Control Enable, ʹ,
												   =1: ȫ˫ģʽ, GMACʹͣ֡ķ; ڰ˫ģʽ, GMACʹܷѹ.  */
	gmac_flowctrl_fcb_bpa	= bit(0),			/* Flow Control Busy/Backpressure Activate, æ/ѹ,
												   =1: ȫ˫ģʽ·ͣ֡ķͻڰ˫ģʽѹ */
} LS1x_gmac_flowctrl_t;

/*
 * VLAN Tag Register 0x001C
 */
typedef enum
{
	gmac_vlan_tag_evt		= bit(16),			/* Enable 12-Bit VLAN Tag Comparison, ʹ12λVLAN TagȽ.
												   =1: ʹ12λVLAN Tagʹ16λVLAN Tag̫֡ȽϺ͹. */
	gmac_vlan_tag_vl_mask	= 0xFFFF,			/* VLAN Tag Identifier for Receive Frames, ֡յVLAN Tagʶ.
												      򱣴802.1QʽVLAN Tag, ڱȽϽյ̫֡λڵ15͵16ֽڵVLAN Tag. */
} LS1x_gmac_vlan_tag_t;

/*
 * GMAC Interrupt Status Register 0x0038
 */
typedef enum
{
	gmac_intstat_mmcerr			= bit(7),		/* MMCУжؼĴκжϲʱ, λΪ1 */
	gmac_intstat_mmctx			= bit(6),		/* MMCжϼĴκжʱ, λΪ1 */
	gmac_intstat_mmcrx			= bit(5),		/* MMCжϼĴκжʱ, λΪ1 */
	gmac_intstat_mmc			= bit(4),		/* MMCж״̬. 7:5κλΪʱ, λΪ1 */
	gmac_intstat_pmt			= bit(3),		/* Դж״̬, PowerDownWakeUpʱ, λΪ1 */
	gmac_intstat_ancomp			= bit(2),		/* RGMII PHYӿԶЭʱ, λΪ1 */
	gmac_intstat_linkstatus		= bit(1),		/* RGMII PHYӿڵ·״̬κα仯ʱ, λΪ1 */
	gmac_intstat_rgmii			= bit(0),		/* RGMII PHYӿڵ·״̬κα仯ʱ, λΪ1 */
} LS1x_gmac_intr_status_t;

/*
 * GMAC Interrupt Mask Register 0x003C
 */
typedef enum
{
	gmac_intmask_timestamp		= bit(9),		/* =1: ֹʱж */
	gmac_intmask_pmt			= bit(3),		/* =1: ֹԴж */
	gmac_intmask_ancomp			= bit(2),		/* =1: ֹPCSԶЭж */
	gmac_intmask_linkchange		= bit(1),		/* =1: ֹPCS·״̬仯ж */
	gmac_intmask_rgmii			= bit(0),		/* =1: ֹRGMIIж */
} LS1x_gmac_intr_mask_t;

/*
 * GMAC Address0 High Register x0040
 */

/*
 * GMAC Address0 Low Register 0x0044
 */

/*
 * GMAC Address1 High Register 0x0048
 */
typedef enum
{
	gmac_addr1_hi_ae			= bit(31),		/* Address Enable ַʹ.
												   =1: ַģʹõ2GMACַȫַ.
												   =0: ַģ鲻ʹõ2GMACַڵַ. */
	gmac_addr1_hi_sa			= bit(30),		/* Source Address ԴGMACַ.
												   =1: GMACַ1ڱȽϽ֡ԴGMACַ. =0: GMACַ1ڱȽϽ֡ĿGMACַ. */
	gmac_addr1_hi_mbc_mask		= (0x3F<<24),	/* bit29~24, Mask Byte Control ģֽڿ.
												      ڱȽÿGMACַֽģλ. 29λĴ18[15:8]ֽ */
	gmac_addr1_hi_mbc_shift 	= 24,
} LS1x_gmac_Addr1_high_t;

/*
 * GMAC Address1 Low Register 0x004C
 */

/*
 * AN Control Register 0x00C0
 */
typedef enum
{
	gmac_AN_ctrl_lr				= bit(17),		/* Lock to Reference οʱ. =1: PHY໷125MHzĲοʱ */
	gmac_AN_ctrl_ecd			= bit(16),		/* Enable Comma Detect ʹͣ̽. =1: ʹPHYͣ̽ͬ */
	gmac_AN_ctrl_ele			= bit(14),		/* External Loopback Enable ⲿʹ. =1: ʹPHY뻷ģʽ */
	gmac_AN_ctrl_enable			= bit(12),		/* Auto-Negotiation Enable ԶЭʹ. =1: GMAC·ԷԶЭ */

	gmac_AN_ctrl_ran			= bit(9),		/* Restart Auto-Negotiation ½ԶЭ. =1: ½ԶЭ */

} LS1x_gmac_AN_control_t;

/*
 * AN Status Register 0x00C4
 */
typedef enum
{
	gmac_AN_stat_es				= bit(8),		/* Extended Status չ״̬. ֻ, ΪGMAC֧չ״̬Ϣ */
	gmac_AN_stat_anc			= bit(5),		/* Auto-Negotiation Complete. ֻ, ָʾԶЭ */
	gmac_AN_stat_ana			= bit(3),		/* Auto-Negotiation Ability ԶЭ. ֻ, ΪGMAC֧ԶЭ */
	gmac_AN_stat_ls				= bit(2),		/* Link Status ·״̬. =1: ָʾ·; =0: ָʾδ */
} LS1x_gmac_AN_status_t;

/*
 * Auto-Negotiation Advertisement Register 0x00C8
 */
typedef enum
{
	gmac_AN_advertise_np		= bit(15),		/* Next Page Support һҳ֧. ֻΪ0, ΪGMAC֧һҳ.  */
	gmac_AN_advertise_rfe_mask	= (0x03<<12),	/* Remote Fault Encoding Զ˴ */
	gmac_AN_advertise_rfe_shift	= 12,			/* 2λָʾ·Զ˷, 뽫IEEE 802.3z37.2.1.5С */
	gmac_AN_advertise_pse_mask	= (0x03<<7),	/* Pause Encoding Pauseλ. IEEE 802.3z37.2.1.4С */
	gmac_AN_advertise_pse_shift = 7,
	gmac_AN_advertise_hd		= bit(6),		/* Half-Duplex ˫. =1: ָʾGMACְ֧˫ */
	gmac_AN_advertise_fd		= bit(5),		/* Full-Duplex ȫ˫. =1: ָʾGMAC֧ȫ˫ */

} LS1x_gmac_AN_advertise_t;

/*
 * Auto-Negotiation Link Partner Ability Register 0x00CC
 */
typedef enum
{
	gmac_AN_lpa_np			= bit(15),		/* Next Page Support һҳ֧. =1: ָʾиһҳϢ; =0: ָʾһҳ潻 */
	gmac_AN_lpa_ack			= bit(14),		/* Acknowledge ȷ. ָʾԶЭ, ·Զ˳ɹյGMACĻҳ */
	gmac_AN_lpa_rfe_mask	= (0x03<<12),	/* Remote Fault Encoding Զ˴. IEEE 802.3z37.2.1.5С */
	gmac_AN_lpa_rfe_shift	= 12,
	gmac_AN_lpa_pse_mask	= (0x03<<7),	/* Pause Encoding Զpause״̬. IEEE 802.3z37.2.14С */
	gmac_AN_lpa_pse_shift	= 7,
	gmac_AN_lpa_hd			= bit(6),		/* Half-Duplex ˫. ָʾԶ˿ڰ˫ģʽ */
	gmac_AN_lpa_fd			= bit(5),		/* Full-Duplex ȫ˫. ָʾԶ˿ȫ˫ģʽ */
} LS1x_gmac_AN_lpa_t;

/*
 * Auto-Negotiation Expansion Register 0x00D0
 */
typedef enum
{
	gmac_AN_exp_npa			= bit(2),		/* Next Page Ability һҳ. ֻΪ0, ΪGMAC֧һҳ */
	gmac_AN_exp_npr			= bit(1),		/* New Page Received յҳ. =1: ָʾGMACյҳ */
} LS1x_gmac_AN_exp_t;

/*
 * SGMII/RGMII Status Register 0x00D8
 */
typedef enum
{
	gmac_mii_linked			= bit(3),		/* Link Status ·״̬. =1: ָʾ·; =0: ָʾ·δ */
	gmac_mii_speed_mask		= (0x03<<1),	/* Link Speed ·ٶ */
	gmac_mii_speed_shift	= 1,
	gmac_mii_speed_2p5MHZ	= 0x00,			/* 00: 2.5MHz */
	gmac_mii_speed_25MHZ	= 0x20,				/* 01: 25MHz */
	gmac_mii_speed_125MHZ	= 0x40,			/* 10: 125MHz */
	gmac_mii_linkmode		= bit(0),		/* Link Mode ·ģʽ */
	gmac_mii_linkmode_hd	= 0,				/* 0: ˫ */
	gmac_mii_linkmode_fd	= 1,				/* 1: ȫ˫ */
} LS1x_gmac_mii_status_t;

/*
 * GMII Address Register 0x0010
 * GMII Data Register 0x0014
 */
typedef enum
{
	gmac_miictrl_phyaddr_mask	= (0x1F<<11),		/* bits15~11, PHY Address. ѡҪ32PHYеĸ */
	gmac_miictrl_phyaddr_shift	= 11,
	gmac_miictrl_gmiireg_mask	= (0x1F<<6),		/* bits10~6, GMII Register. ѡҪʵĵPHYĸGMIIüĴ */
	gmac_miictrl_gmiireg_shift	= 6,
	gmac_miictrl_csr_mask		= (0x07<<2),		/* bits4~2, CSR Clock Range. MDCʱclk_csr_iʱƵʱ */
	gmac_miictrl_csr_shift		= 2,				/* (CR)CSR Clock Range: */
	gmac_miictrl_csr_5			= 0x00000014,		/* 250-300 MHz */
	gmac_miictrl_csr_4			= 0x00000010,		/* 150-250 MHz */
	gmac_miictrl_csr_3			= 0x0000000C,		/* 5-60 MHz   */
	gmac_miictrl_csr_2			= 0x00000008,		/* 20-35 MHz   */
	gmac_miictrl_csr_1			= 0x00000004,		/* 100-150 MHz */
	gmac_miictrl_csr_0			= 0x00000000,		/* 60-100 MHz  */
	gmac_miictrl_wr				= bit(1),			/* GMII Write. =1: ͨGMII ݼĴPHYд, =0: ͨGMIIݼĴPHYж */
	gmac_miictrl_busy			= bit(0),			/* GMII Busy. ԼĴ4ͼĴ5д֮ǰ, λӦΪ0. дĴ4֮ǰλ0.
                                                       ڷPHY ļĴʱ, ӦóҪλΪ1, ʾGMIIӿд߶ڽ. */

	gmac_miidata_mask			= 0xFFFF,			/* 򱣴˶PHYйʲ16λ, ߶PHYйдʵ16λ. */

} LS1x_gmac_miictrl_t;

#ifdef LS1C

/******************************************************************************
 * IEEE1588 Registers of GMAC
 ******************************************************************************/

typedef struct LS1C_gmac_ieee1588_s
{
	volatile unsigned int stamp_ctrl;					/* 0xBFE10700 Time Stamp Control Register */
	volatile unsigned int sub_second_inc; 				/* 0xBFE10704 Sub-Second Increment Register */
	volatile unsigned int systm_second; 				/* 0xBFE10708 System Time - Seconds Register */
	volatile unsigned int systm_nanosecond; 			/* 0xBFE1070C System Time - Nanoseconds Register */
	volatile unsigned int systm_second_upd; 			/* 0xBFE10710 System Time - Seconds Update Register */
	volatile unsigned int systm_nanosecond_upd; 		/* 0xBFE10714 System Time - Nanoseconds Update Register */
	volatile unsigned int ts_addend;	 				/* 0xBFE10718 Time Stamp Addend Register */
	volatile unsigned int tgttm_second; 				/* 0xBFE1071C Target Time Seconds Register */
	volatile unsigned int tgttm_nanosecond; 			/* 0xBFE10720 Target Time Nanoseconds Register */
	volatile unsigned int systm_hisecond; 				/* 0xBFE10724 System Time - Higher Word Seconds Register */
	volatile unsigned int ts_status; 					/* 0xBFE10728 Time Stamp Status Register */
	volatile unsigned int pps_auxts_nanosecond; 		/* 0xBFE1072C PPS Auxiliary Time Stamp - Nanoseconds Register */
	volatile unsigned int pps_auxts_second; 			/* 0xBFE10730 PPS Auxiliary Time Stamp - Seconds Register */
} LS1C_gmac_ieee1588_t;

#endif

/******************************************************************************
 * DMA registers of GMAC
 ******************************************************************************/

#define LS1x_GDMA0_BASE			0xBFE11000
#if defined(LS1B)
#define LS1B_GDMA1_BASE			0xBFE21000
#endif

typedef struct LS1x_gdma_regs
{
	volatile unsigned int busmode; 			/* 0xBFE11000 Bus Mode */
	volatile unsigned int txpoll; 			/* 0xBFE11004 Transmit Poll Demand */
	volatile unsigned int rxpoll; 			/* 0xBFE11008 Receive Poll Demand */
	volatile unsigned int rxdesc0; 			/* 0xBFE1100C Start of Receive Descriptor List Address */
	volatile unsigned int txdesc0; 			/* 0xBFE11010 Start of Transmit Descriptor List Address */
	volatile unsigned int status; 			/* 0xBFE11014 Status */
	volatile unsigned int control; 			/* 0xBFE11018 Operation Mode */
	volatile unsigned int intenable; 		/* 0xBFE1101C Interrupt Enable */
	volatile unsigned int mfbocount; 		/* 0xBFE11020 Missed Frame and Buffer Overflow Counter */
#ifdef LS1C
	volatile unsigned int rxintwdtmr; 		/* 0xBFE11024 Receive Interrupt Watchdog Timer Register */
	volatile unsigned int axibusmode; 		/* 0xBFE11028 AXI Bus Mode Register */
	volatile unsigned int axistatus; 		/* 0xBFE1102C AXI Status Register */
	volatile unsigned int rsv2[6];
#else
	volatile unsigned int rsv1[9];          /* XXX LS1B no WD & AXI Register */
#endif
	volatile unsigned int curtxdesc; 		/* 0xBFE11048 Current Host Transmit Descriptor */
	volatile unsigned int currxdesc; 		/* 0xBFE1104C Current Host Receive Descriptor */
	volatile unsigned int curtxbuf; 		/* 0xBFE11050 Current Host Transmit Buffer Address */
	volatile unsigned int currxbuf; 		/* 0xBFE11054 Current Host Receive Buffer Address */
} LS1x_gdma_regs_t;

/*
 * Bus Mode Register of GMAC's DMA 	Offset: 0x00
 */
typedef enum
{
	gdma_busmode_mb			= bit(26),		/* Mixed Burst ͻ *//* ûùĴλ */
	gdma_busmode_aal		= bit(25),		/* Address-Aligned Beats ַ *//* ûùĴλ */
	gdma_busmode_8xpbl		= bit(24),		/* 8XPBL Mode ǷʹPBLX8ģʽ *//* ûùĴλ  */
	gdma_busmode_usp		= bit(23),		/* Use Separate PBL ʹ÷PBLֵ *//* ûùĴλ  */
	gdma_busmode_rpbl_mask	= (0x3F<<17),	/* RxDMA PBL RxDMAͻ䳤 */
	gdma_busmode_rpbl_shift = 17,			/* ʾһRxDMAͻ䳤. ֻΪ1,2,4,8,1632, ֵЧ*/
	gdma_busmode_fb			= bit(16),		/* Fixed Burst ͻ䳤 *//* ûùĴλ */
	gdma_busmode_pr_mask	= (0x03<<14),	/* Rx:Tx priority ratio, RxDMATxDMAȼ, DAλΪ1ʱ  */
	gdma_busmode_pr_shift   = 14,
	gdma_busmode_pr_1		= (0<<14),		/* (PR)TX:RX DMA priority ratio 1:1 */
	gdma_busmode_pr_2		= bit(14),		/* (PR)TX:RX DMA priority ratio 2:1 */
	gdma_busmode_pr_3		= (2<<14),		/* (PR)TX:RX DMA priority ratio 3:1 */
	gdma_busmode_pr_4		= (3<<14),		/* (PR)TX:RX DMA priority ratio 4:1 */
	gdma_busmode_pbl_mask	= (0x3f<<8),	/* Programmable Burst Length ɱͻ䳤 *//* ûùĴλ */
	gdma_busmode_pbl_shift	= 8,
	gdma_busmode_pbl_256	= 0x01002000,	/* (DmaBurstLengthx8 | DmaBurstLength32) = 256 */
	gdma_busmode_pbl_128	= 0x01001000,	/* (DmaBurstLengthx8 | DmaBurstLength16) = 128 */
	gdma_busmode_pbl_64		= 0x01000800,	/* (DmaBurstLengthx8 | DmaBurstLength8)  = 64 */
	gdma_busmode_pbl_32		= bit(13), 		/* Dma burst length = 32 */
	gdma_busmode_pbl_16		= bit(12), 		/* Dma burst length = 16 */
	gdma_busmode_pbl_8		= bit(11), 		/* Dma burst length = 8 */
	gdma_busmode_pbl_4		= bit(10), 		/* Dma burst length = 4 */
	gdma_busmode_pbl_2		= bit(9),		/* Dma burst length = 2 */
	gdma_busmode_pbl_1		= bit(8),		/* Dma burst length = 1 */
	gdma_busmode_pbl_0		= 0,			/* Dma burst length = 0 */
	gdma_busmode_atds		= bit(7),		/* Alternate Descriptor size, =1: ʹ32ֽڴС, =0: ʹ16ֽڴС */
	gdma_busmode_dsl_mask	= (0x1F<<2),	/* Descriptor Skip Length, 2ľ. ֵΪ0ʱ, ĬΪDMAС */
	gdma_busmode_dsl_shift  = 2,
	gdma_busmode_dsl_16		= bit(6),
	gdma_busmode_dsl_8		= bit(5),
	gdma_busmode_dsl_4		= bit(4),
	gdma_busmode_dsl_2		= bit(3),
	gdma_busmode_dsl_1		= bit(2),
	gdma_busmode_dsl_0		= 0,
	gdma_busmode_das		= bit(1),		/* DMA Arbitration scheme, =0: RxDMATxDMAתٲû, =1: RxDMAȼTxDMAȼ(PRֵ) */
	gdma_busmode_swreset	= bit(0),		/* λøDMAλGMACڲĴ߼. λʱλԶ */
} LS1x_gdma_busmode_t;

/*
 * Status Register of GMAC's DMA 	Offset: 0x14
 */
typedef enum
{
	gdma_status_tti			= bit(29),		/* Time-Stamp Trigger Interrupt, ʱģ鴥ж. ֻ */
	gdma_status_gpi			= bit(28),		/* GMAC PMT Interrupt, Դģ鴥ж. ֻ */
	gdma_status_gmi			= bit(27),		/* GMAC MMC Interrupt, MMCģ鴥ж. ֻ */
	gdma_status_gli			= bit(26),		/* GMAC Line interface Interrupt, GMACģPCSRGMIIģ鴥ж. ֻ */
	gdma_status_eb_mask		= (0x07<<23),	/* Error Bits, λ(ο) */
	gdma_status_eb_shift    = 23,
	gdma_status_eb_25		= bit(25),		/* =1: ʴ; =0: ݻʴ */
	gdma_status_eb_24		= bit(24),		/* =1: ; =0: д */
	gdma_status_eb_23		= bit(23),		/* =1: TxDMAݴз; =0: RxDMAݴз */
	gdma_status_txs_mask	= (0x07<<20),	/* Transmit Process State, ״̬(ο) */
	gdma_status_txs_shift   = 20,
	gdma_status_txs_0		= (0<<20),		// 000: ֹͣ; λֹͣ
	gdma_status_txs_1		= (1<<20),		// 001: ڽ; ȡ
	gdma_status_txs_2		= (2<<20),		// 010: ڽ; ȴ״̬
	gdma_status_txs_3		= (3<<20),		// 011: ڽ; ӷͻȡݲ͵FIFO(TxFIFO)
	gdma_status_txs_4		= (4<<20),		// 100: дʱ״̬
	gdma_status_txs_5		= (5<<20),		// 101: 
	gdma_status_txs_6		= (6<<20),		// 110: ; ûߴ仺.
	gdma_status_txs_7		= (7<<20),		// 111: ; رմ.
	gdma_status_rxs_mask	= (0x07<<17),	/* bits: 19-17, Receive Process State, չ״̬(ο) */
	gdma_status_rxs_shift   = 17,
	gdma_status_rxs_0		= (0<<17),		// 000: ֹͣ; λ߽յֹͣ
	gdma_status_rxs_1		= (1<<17),		// 001: ; ȡ.
	gdma_status_rxs_2		= (2<<17),		// 010: ;
	gdma_status_rxs_3		= (3<<17),		// 011: ; ȴհ.
	gdma_status_rxs_4		= (4<<17),		// 100: ͣ; .
	gdma_status_rxs_5		= (5<<17),		// 101: ; رս.
	gdma_status_rxs_6		= (6<<17),		// 110: ʱд״̬.
	gdma_status_rxs_7		= (7<<17),		// 111: ; ݴӽջ洫䵽ϵͳڴ.
	gdma_status_nis			= bit(16),		/* Normal Interrupt Summary, жϻ, ʾϵͳǷж */
	gdma_status_ais			= bit(15),		/* Abnormal Interrupt Summary, 쳣жϻ, ʾϵͳǷ쳣ж */
	gdma_status_erxi		= bit(14),		/* Early Receive Interrupt, ǰж, ʾDMAѾѰĵһдջ */
	gdma_status_fbei		= bit(13),		/* Fatal Bus Error Interrupt, ߴж, ʾߴ, Ϣ[25:23]. λúDMAֹͣ߷ʲ */
	gdma_status_etxi		= bit(10),		/* Early Transmit Interrupt, ǰж, ʾҪ̫֡Ѿȫ䵽MTLģеĴ FIFO */
	gdma_status_rxwt		= bit(9),		/* Receive Watchdog Timeout, տŹʱ, ʾյһС2048ֽڵ̫֡ */
	gdma_status_rxstop		= bit(8),		/* Receive Process Stopped, չֹͣ */
	gdma_status_rxbufu		= bit(7),		/* Receive Buffer Unavailable, ջ治 */
	gdma_status_rxi			= bit(6),		/* Receive Interrupt, ж, ָʾ֡, ֡յ״̬ϢѾд, մ״̬ */
	gdma_status_txunf		= bit(5),		/* Transmit Underflow, 仺, ָʾ֡͹вջ */
	gdma_status_rxovf		= bit(4),		/* Receive Overflow, ջ, ָʾ֡չнջ */
	gdma_status_txjt		= bit(3),		/* Transmit Jabber Timeout */
	gdma_status_txbufu		= bit(2),		/* Transmit Buffer Unavailable, 仺治, ʾбеһܱDMA */
	gdma_status_txstop		= bit(1),		/* Transmit Process Stopped, ֹͣ */
	gdma_status_txi			= bit(0),		/* Transmit Interrupt, ж, ʾ֡ɲҵһ31λλ */
} LS1x_gdma_status_t;

/*
 * Operation Mode Register of GMAC's DMA 	Offset: 0x18
 */
typedef enum
{
	gdma_ctrl_droptcpcse	= bit(26),		/* رնTCP/IP Checksum̫֡Ĺ, =1: GMACchecksum̫֡ */
	gdma_ctrl_rxsf			= bit(25),		/* Receive Store and Forward, մ洢ת, =1: MTLģֻѾȫ洢ڽFIFOе̫֡ */
	gdma_ctrl_disfrxf		= bit(24),		/* Disable Flushing of Received Frames, رճˢյ̫֡Ĺ,
	 	 	 	 	 	 	 	 	 	 	   =1: DMAڽ߽ջ治ʱˢκ̫֡ */
	gdma_ctrl_RFA_2			= bit(23),		/* RFA[2]: MSB of Threshold for Activating Flow Control ֵ */
	gdma_ctrl_RFD_2			= bit(22),		/* RFD[2]: MSB of Threshold for Deactivating Flow Control رֵ */
	gdma_ctrl_txsf			= bit(21),		/* Transmit Store and Forward, ʹ洢ת, =1: ֡ķֻ֡ѾȫMTLĴFIFO */
	gdma_ctrl_ftxfifo		= bit(20),		/* Flush Transmit FIFO, ˢFIFO, =1: ߼λΪĬֵ, һᵼ·FIFOȫʧ */
	gdma_ctrl_ttc_mask		= (0x7<<14),	/* Transmit Threshold Control, ֵ, ֡СֵʱMTLᴫ֡ */
	gdma_ctrl_ttc_shift		= 14,
	gdma_ctrl_ttc_64		= (0<<14),		// 000: 64  ֽ
	gdma_ctrl_ttc_128		= (1<<14),		// 001: 128 ֽ
	gdma_ctrl_ttc_192		= (2<<14),		// 010: 192 ֽ
	gdma_ctrl_ttc_256		= (3<<14),		// 011: 256 ֽ
	gdma_ctrl_ttc_40		= (4<<14),		// 100: 40  ֽ
	gdma_ctrl_ttc_32		= (5<<14),		// 101: 32  ֽ
	gdma_ctrl_ttc_24		= (6<<14),		// 110: 24  ֽ
	gdma_ctrl_ttc_16		= (7<<14),		// 111: 16  ֽ
	gdma_ctrl_txstart		= bit(13),		/* Start/Stop Transmission Command, =1: ״̬, =0: ֹͣ״̬ */
	gdma_ctrl_RFD			= (0x03<<11),	/* RFD: Threshold for deactivating flow control رֵ */
	gdma_ctrl_RFA			= (0x03<<9),	/* RFA:Threshold for Activating flow control ֵ */
	gdma_ctrl_enhwfc		= bit(8),		/* Enable HW flow control, =1: ڽFIFOʵӲص·Ч */
	gdma_ctrl_ferrf			= bit(7),		/* Forward Error Frames, ֡, =1: մ֡(֡:CRC,ͻ,֡,Źʱ,) */
	gdma_ctrl_fugf			= bit(6),		/* Forward Undersized Good Frames, =1: FIFOûдС64ֽڵ̫֡ */
	gdma_ctrl_rtc_mask		= (0x3<<3),		/* Receive Threshold Control, ֵ, ֡СֵʱMTLո֡ */
	gdma_ctrl_rtc_shift		= 3,
	gdma_ctrl_rtc_64		= (0<<3),		// 00: 64  ֽ
	gdma_ctrl_rtc_32		= (1<<3),		// 01: 32  ֽ
	gdma_ctrl_rtc_96		= (2<<3),		// 10: 96  ֽ
	gdma_ctrl_rtc_128		= (3<<3),		// 11: 128 ֽ
	gdma_ctrl_txopsecf		= bit(2),		/* TX Operate on Second Frame, =1: DMAڵһ̫֡״̬δдʱԿʼڶ̫֡ */
	gdma_ctrl_rxstart		= bit(1),		/* Start/Stop Receive, =1: ս״̬, =0: սֹͣ״̬ */
} LS1x_gdma_ctrl_t;

/*
 * Interrupt Enable Register of GMAC's DMA 	Offset: 0x1C
 */
typedef enum
{
	gdma_ienable_nis		= bit(16),		/* Normal Interrupt Summary Enable, =1: жʹ, =0: жϲʹ */
	gdma_ienable_ais		= bit(15),		/* Abnormal Interrupt Summary Enable, =1: жʹ, =0: жϲʹ */
	gdma_ienable_erxi		= bit(14),		/* Early Receive Interrupt Enable, ڽжʹ, =1: ڽжʹ */
	gdma_ienable_fbei		= bit(13),		/* Fatal Bus Error Enable, =1: жʹ */
	gdma_ienable_etxi		= bit(10),		/* Early Transmit Interrupt Enable, =1: ʹڴж */
	gdma_ienable_rxwdog		= bit(9),		/* Receive Watchdog Timeout Enable, =1: ʹܽտŹʱж */
	gdma_ienable_rxstop		= bit(8),		/* Receive Stopped Enable, =1: ʹֹܽͣж */
	gdma_ienable_rxbufu		= bit(7),		/* Receive Buffer Unavailable Enable, =1: ʹܽջж */
	gdma_ienable_rxi		= bit(6),		/* Receive Interrupt Enable, =1: ʹܽж */
	gdma_ienable_txunf		= bit(5),		/* Underflow Interrupt Enable, =1: ʹܴFIFOж */
	gdma_ienable_rxovf		= bit(4),		/* Overflow Interrupt Enable, =1: ʹܽFIFOж */
	gdma_ienable_txjt		= bit(3),		/* Transmit Jabber Timeout Enable, =1: ʹJabberʱж */
	gdma_ienable_txbufu		= bit(2),		/* Transmit Buffer Unavailable Enable, =1: ʹܴ仺治ж */
	gdma_ienable_txstop		= bit(1),		/* Transmit Stopped Enable, =1: ʹֹܴͣж */
	gdma_ienable_txi		= bit(0),		/* Transmit Interrupt Enable, =1: ʹܴж */

	gdma_ien_base = (gdma_ienable_nis | gdma_ienable_ais | gdma_ienable_fbei),
	gdma_ien_rx	  = (gdma_ienable_rxstop | gdma_ienable_rxi | gdma_ienable_rxbufu),
	gdma_ien_tx	  = (gdma_ienable_txstop | gdma_ienable_txi | gdma_ienable_txunf),

} LS1x_gdma_intr_en_t;

/*
 * Missed Frame and Buffer Overflow Counter Register of GMAC's DMA 	Offset: 0x20
 */
typedef enum
{
	gdma_mfbocnt_ovfcnt			= bit(28),		/* FIFO ָʾλ */
	gdma_mfbocnt_fmcnt_mask		= (0x07FF<<17),	/* ָʾӦóʧ֡ĸ */
	gdma_mfbocnt_fmcnt_shift    = 17,
	gdma_mfbocnt_ovfmcnt		= bit(16),		/* ʾʧ֡Ѿֵ */
	gdma_mfbocnt_rxufmcnt_mask	= (0xFFFF<<0),	/* ָʾΪջ治õ֡ʧļ */
} LS1x_gdma_mfbo_cnt_t;

#ifdef LS1C

/*
 * Receive Interrupt Watchdog Timer Register
 */
typedef enum
{
	gdma_rxintwdtmr_riwt_mask	= 0xFF,			/* RI Watchdog Timer count, տŹʱ */
	gdma_rxintwdtmr_riwt_shift  = 0,			/* Źú, ʾʱx256ʱΪλʱ.
												   DMAյݰstatusĴRIλΪ0ʱʼʱ,
												   ŹʱRIλ1. RIλΪ1λ */
} LS1C_gdma_rxintwdtmr_t;

/*
 * AXI Bus Mode Register
 */
typedef enum
{
	gdma_axibusmode_en_lpi		= bit(31),		/* Enable LPI (Low Power Interface), õ͹Ľӿ */
	gdma_axibusmode_unlock		= bit(30), 		/* Unlock on Magic Packet or Remote Wake Up, ħԶ̻Ѱ */
												/* =1: GMACڵ͹(Low Power)״̬ʱֻͨħԶ̻Ѱ»ص״̬;
												 * =0: GMACڵ͹(Low Power)״̬ʱֻͨ»ص״̬
												 */
	gdma_axibusmode_wrlmt_mask	= (0x07<<20),	/* AXI Maximum Write Out Standing Request Limit, AXIoutstandingд */
	gdma_axibusmode_wrlmt_shift = 20,			/* λAXIӿдoutstanding */
	gdma_axibusmode_rdlmt_mask	= (0x07<<16),	/* AXI Maximum Read Out Standing Request Limit, AXIoutstanding */
	gdma_axibusmode_rdlmt_shift = 16,			/* λAXIӿڶoutstanding */
	gdma_axibusmode_addr_align  = bit(12),		/* Address-Aligned Beats, AXIַ, ֻ */
	gdma_axibusmode_blen_256	= bit(7),		/* AXI Burst Length 256, AXI Burst256 */
	gdma_axibusmode_blen_128	= bit(6),
	gdma_axibusmode_blen_64		= bit(5),
	gdma_axibusmode_blen_32		= bit(4),
	gdma_axibusmode_blen_16		= bit(3),
	gdma_axibusmode_blen_8		= bit(2),
	gdma_axibusmode_blen_4		= bit(1),
	gdma_axibusmode_blen_undef	= bit(0),		/* AXI Undefined Burst Length, δAXI Burst */
} LS1C_gdma_axibusmode_t;

/*
 * AXI Status Register
 */
typedef enum
{
	gdma_axistatus_rd_request	= bit(1),		/* =1: ʾǰAXIڷͶ */
	gdma_axistatus_wr_request	= bit(0),		/* =1: ʾǰAXIڷд */
} LS1C_gdma_axistatus_t;

#endif

/******************************************************************************
 * Receive Descriptor RDES0 - Status
 */

typedef enum
{
	gdma_rxdesc_stat_own		= bit(31),		/* OWN, =1: ǰDMA, =0: . DMAģһδʱ, Ὣλ0 */
	gdma_rxdesc_stat_afm		= bit(30),		/* Destination Address Filter Fail, Ŀַ˴. =1: ǰ֡ĿַGMACڲ֡Ŀַ */
	gdma_rxdesc_stat_fr_mask    = (0x3FFF<<16), /* bit29~16, Frame length ֡, ʾյǰ֡ĳ, ESλΪ0ʱЧ */
	gdma_rxdesc_stat_fr_shift   = 16,
	gdma_rxdesc_stat_es      	= bit(15),		/* Error Summary Ϣ, ָʾǰ֡Ƿ, ֵΪRDES[0,1,3,4,6,7,11,14]λ(OR)Ľ */
	gdma_rxdesc_stat_de			= bit(14),		/* Descriptor Error , =1: ǰָbuffer֡OWNΪ0() */
	gdma_rxdesc_stat_saf		= bit(13),		/* Source Address Filter Fail Դַ˴, =1: ǰ֡ԴַGMACڲ֡Դַ */
	gdma_rxdesc_stat_le			= bit(12),		/* Length Error ȴ, =1: ǰ֡ĬϳȲ. Frame TypeλΪ1CRC ErrorλΪ0ʱЧ */
	gdma_rxdesc_stat_oe			= bit(11),		/* Over Flow Error , =1: ո֡ʱGMACڲRxFIFO */
	gdma_rxdesc_stat_vlan		= bit(10),		/* VLAN Tag VLAN־, =1: ֡ΪVLAN */
	gdma_rxdesc_stat_fs			= bit(9),		/* First Desciptor һ, =1: ǰָbufferΪǰ֡ĵһbuffer */
	gdma_rxdesc_stat_ls			= bit(8),		/* Last Desciptor һ, =1: ǰָbufferΪǰ֡һbuffer */
	gdma_rxdesc_stat_ipce_gf	= bit(7),		/* IPC Checksum Error/Giant Frame У/֡.
	 	 	 	 	 	 	 	 	 	 	 	   =1: IPCУ鹦ʾǰ֡IPv4ͷУֵ֡ڲУֵ.
	 	 	 	 	 	 	 	 	 	 	 	 	   δʾǰ֡Ϊһ֡(ȴ1518ֽ) */
	gdma_rxdesc_stat_lc			= bit(6),		/* Late Collision ڳͻ, =1: ڰ˫ģʽ, ǰ֡ʱһڳͻ */
	gdma_rxdesc_stat_ft			= bit(5),		/* Frame Type ֡, =1: ǰ֡Ϊһ̫ʽ֡, =0: ǰ֡ΪһIEEE802.3ʽ֡ */
	gdma_rxdesc_stat_rwt		= bit(4),		/* Receive Watchdog Timeout, =1: ǰʱֵ˽ģ鿴Ź·ʱӵֵ, Ƚ֡ʱ */
	gdma_rxdesc_stat_re			= bit(3),		/* Receive Error մ, =1: յǰ֡ʱڲģ. ڲźrxer1rxdv1 */
	gdma_rxdesc_stat_dbe		= bit(2),		/* Dribble bit Error λ, =1: ֡Ȳ, ܳΪλ, λֻmiiģʽЧ */
	gdma_rxdesc_stat_ce			= bit(1),		/* CRC Error CRCУ, =1: յǰ֡ʱڲCRCУ. λֻlast descriptor(RDES0[8])Ϊ1ʱЧ */
	gdma_rxdesc_stat_rmpce		= bit(0),		/* RX GMAC Checksum/payload Checksum Error У/У.
												   =1: յǰ֡ʱڲRX GMACĴ1-15дһƥ䵱ǰ֡Ŀĵַ.
												   =0: RX GMAC Ĵ0ƥ֡Ŀĵַ. Full Checksum Offload Engineʱ,
												   =1: ֡TCP/UDP/ICMPУ. λΪ1ʱҲܱʾǰ֡ʵʽܳ֡ڲسȲ. */
} LS1x_gdma_rxdesc_stat_t;

/******************************************************************************
 * Receive Descriptor RDES1 - Control, Address
 */
typedef enum
{
	gdma_rxdesc_ctrl_di			= bit(31),		/* 1: Disable Interrupt in Completion */
	gdma_rxdesc_ctrl_userdef	= bit(30),		/* XXX Զ, ѭ */
	gdma_rxdesc_ctrl_rer		= bit(25),		/* Receive End of Ring, =1: Ϊһ, һĵַΪĻַ */
	gdma_rxdesc_ctrl_rch		= bit(24),		/* Second Address Chained, =1: еĵڶbufferַָһĵַ */
	gdma_rxdesc_ctrl_rbs2_mask	= (0x07FF<<11),	/* bits: 21-11, Receive Buffer Size 2, ʾbuffer2ĴС */
	gdma_rxdesc_ctrl_rbs2_shift = 11,
	gdma_rxdesc_ctrl_rbs1_mask	= (0x07FF<<0),	/* bits: 10-0, Receive Buffer Size 1, ʾbuffer1ĴС */
} LS1x_gdma_rxdesc_ctrl_t;

/******************************************************************************
 * Transmit Descriptor TDES0 - Status
 */
typedef enum
{
	gdma_txdesc_stat_own		= bit(31),		/* OWN, =1: ʾǰDMA, 0ʾ. DMAģһδʱ, Ὣλ0 */
	gdma_txdesc_stat_ttss		= bit(17),		/* Tx Time Stamp Status, IEEE1588ʱ, =1: TDES2TDES3б˸÷֡ʱϢ. λ */
	gdma_txdesc_stat_ihe		= bit(16),		/* IP Header Error, =1: ʾڲУģ鷢ָ÷֡IPͷ, ҲԸκ޸ */
	gdma_txdesc_stat_es			= bit(15),		/* Error Summary, ָʾǰ֡Ƿ, ֵΪTDES[1,2,8,9,10,11,13,14]λ(OR)Ľ */
	gdma_txdesc_stat_jt			= bit(14),		/* Jabber Timeout, =1: ʾGMACģJabberʱ */
	gdma_txdesc_stat_ff			= bit(13),		/* Frame Flushed, =1: ʾһˢDMA/MTLڲ֡ˢµ */
	gdma_txdesc_stat_pce		= bit(12),		/* Payload Checksum Error, =1: ʾڲУģ֡вУʱ. Уģʱ, λЧ */
	gdma_txdesc_stat_lc			= bit(11),		/* Loss of Carrier, =1: ʾڷ͸֡زʧ(gmii_crsźŶδ) */
	gdma_txdesc_stat_nc			= bit(10),		/* No Carrier, =1: ʾڷ͹, PHYزźһֱδ */
	gdma_txdesc_stat_lco		= bit(9),		/* Late Collision, =1: ʾڰ˫ģʽ, ǰ֡ʱһڳͻ */
	gdma_txdesc_stat_ec			= bit(8),		/* Excessive Collison, =1: ʾڷ͵ǰ֡ʱ16γͻ */
	gdma_txdesc_stat_vf			= bit(7),		/* VLAN Frame, =1: ʾǰ֡ΪһVLAN֡ */
	gdma_txdesc_stat_cc_mask	= (0x0F<<3),	/* bits: 6-3, Collsion Count, ʾǰ֡ڳɹ֮ǰͻ */
	gdma_txdesc_stat_cc_shift   = 3,
	gdma_txdesc_stat_ed			= bit(2),		/* Excessive Deferral, =1: ʾǰ֡ */
	gdma_txdesc_stat_uf			= bit(1),		/* Underflow Error, =1: ʾǰ֡ʱ, ݴbufferС򲻿 */
	gdma_txdesc_stat_db			= bit(0),		/* Defered Bit, =1: ʾ˴ηͱӳ, ֻڰ˫ģʽЧ */
} LS1x_gdma_txdesc_stat_t;

/******************************************************************************
 * Transmit Descriptor TDES1 - Control, Address
 */
typedef enum
{
	gdma_txdesc_ctrl_ic			= bit(31),		/* Interrption on Complete, =1: ʾ֡ӷɺ󽫻STATUSĴTIλ(CSR5[0]) */
	gdma_txdesc_ctrl_ls			= bit(30),		/* Last Segment, =1: ʾǰbufferһ֡ݵһ(֡Ϊ) */
	gdma_txdesc_ctrl_fs			= bit(29),		/* First Segment, =1: ʾǰbufferһ֡ݵĵһ(֡Ϊ) */
	gdma_txdesc_ctrl_cic_mask	= (0x02<<27),	/* bits: 28-27, Checksum Insertion Control, ڲģǷڷ֡У */
	gdma_txdesc_ctrl_cic_shift  = 27,
	gdma_txdesc_ctrl_cic_ipv4	= bit(27),
	gdma_txdesc_ctrl_cic_tcp	= bit(28),
	gdma_txdesc_ctrl_cic_all	= (0x02<<27),
	gdma_txdesc_ctrl_dc			= bit(26),		/* =1, Disable CRC, =1: GMACӲÿ֡ĽβCRCУ */
	gdma_txdesc_ctrl_ter		= bit(25),		/* Transmit End of Ring, =1: ʾΪһ, һĵַΪĻַ */
	gdma_txdesc_ctrl_tch		= bit(24),		/* Second Address Chained, =1: ʾеĵڶbufferַָһĵַ  */
	gdma_txdesc_ctrl_dp			= bit(23),		/* Dissable Pading, =1: ʾGMACԳС64ֽڵݰп */
	gdma_txdesc_ctrl_ttse		= bit(22),		/* Transmit Time Stamp Enable, =1: ʾڲģIEEE1588Ӳʱ, TDES1[29]Ϊ1ʱЧ */
	gdma_txdesc_ctrl_tbs2_mask	= (0x07FF<<11),	/* bits: 21-11, Transmit Buffer Size 2, ʾbuffer2ĴС. TDES1[24]Ϊ1ʱ, Ч */
	gdma_txdesc_ctrl_tbs2_shift = 11,
	gdma_txdesc_ctrl_tbs1_mask	= (0x07FF<<0),	/* bits: 10-0, Transmit Buffer Size 1, ʾbuffer1ĴС. һֱЧ */
} LS1x_txdesc_ctrl_t;

/*
 * GMAC DMA Receive and Transmit Descriptor
 * __attribute__((packed)): ߱ȡṹڱеŻ,ʵռֽж---GCCе﷨
 */

typedef struct gmac_dma_desc
{
	volatile unsigned int status;     	/* Status */
	volatile unsigned int control;		/* 31-22: Control; 21-11: Buffer 2 lenght; 10-0: Buffer 1 length */
	volatile unsigned int bufptr;		/* Buffer 1 pointer */
	volatile unsigned int nextdesc;		/* Next descriptor pointer (Dma-able) in chain structure */
	volatile unsigned int temp1;
	volatile unsigned int temp2;
	volatile unsigned int temp3;
	volatile unsigned int temp4;		/* Total: 32 Bytes */
} __attribute__((packed)) LS1x_gdma_desc_t;

typedef LS1x_gdma_desc_t 	LS1x_rxdesc_t;
typedef LS1x_gdma_desc_t 	LS1x_txdesc_t;

/*
 * mii status defination
 */
typedef enum
{
	LINKDOWN	= 0,
	LINKUP		= 1,
} mii_linkstate;

typedef enum
{
	HALFDUPLEX	= 1,
	FULLDUPLEX	= 2,
} mii_duplexmode;

typedef enum
{
	SPEED10     = 1,
	SPEED100    = 2,
	SPEED1000   = 3,
} mii_speed;

/*
 * mac-dma desc mode
 */
typedef enum
{
	RINGMODE	= 0,
	CHAINMODE	= 1,
} gdma_descmode;

/******************************************************************************
 * macro for operate
 ******************************************************************************/

/*
 * start gmac-dma
 */
#define GDMA_START_RX(hwGDMA) \
        do { \
            hwGDMA->control |= gdma_ctrl_rxstart; \
            ls1x_sync(); \
        } while (0)

#define GDMA_START_TX(hwGDMA) \
        do { \
            hwGDMA->control |= gdma_ctrl_txstart; \
            ls1x_sync(); \
        } while (0)

#define GDMA_START(hwGDMA) \
        do { \
            hwGDMA->control |= (gdma_ctrl_txstart | gdma_ctrl_rxstart); \
            ls1x_sync(); \
        } while (0)

/*
 * stop gmac-dma
 */
#define GDMA_STOP_RX(hwGDMA) \
        do { \
            hwGDMA->control &= ~gdma_ctrl_rxstart; \
            ls1x_sync(); \
        } while (0)

#define GDMA_STOP_TX(hwGDMA) \
        do { \
            hwGDMA->control &= ~gdma_ctrl_txstart; \
            ls1x_sync(); \
        } while (0)

#define GDMA_STOP(hwGDMA) \
        do { \
            hwGDMA->control &= ~(gdma_ctrl_txstart | gdma_ctrl_rxstart); \
            ls1x_sync(); \
        } while (0)

/*
 * gmac-dma interrupt
 */
#define GDMA_INT_EN(hwGDMA) \
        do { \
            hwGDMA->intenable = (gdma_ien_base | gdma_ien_rx | gdma_ien_tx); \
            ls1x_sync(); \
        } while (0)

#define GDMA_INT_DIS(hwGDMA) \
        do { \
            hwGDMA->intenable = 0; \
            ls1x_sync(); \
        } while (0)

/*
 * gmac start
 */
#define GMAC_START_TX(hwGMAC) \
        do { \
            hwGMAC->control |= gmac_ctrl_te; \
            ls1x_sync(); \
        } while (0)

#define GMAC_START_RX(hwGMAC) \
        do { \
            hwGMAC->control |= gmac_ctrl_re; \
            ls1x_sync(); \
        } while (0)

#define GMAC_START(hwGMAC) \
        do { \
            hwGMAC->control |= (gmac_ctrl_te | gmac_ctrl_re); \
            ls1x_sync(); \
        } while (0)

/*
 * gmac stop
 */
#define GMAC_STOP_TX(hwGMAC) \
        do { \
            hwGMAC->control &= ~gmac_ctrl_te; \
            ls1x_sync(); \
        } while (0)

#define GMAC_STOP_RX(hwGMAC) \
        do { \
            hwGMAC->control &= ~gmac_ctrl_re; \
            ls1x_sync(); \
        } while (0)

#define GMAC_STOP(hwGMAC) \
        do { \
            hwGMAC->control &= ~(gmac_ctrl_te | gmac_ctrl_re); \
            ls1x_sync(); \
        } while (0)

#ifdef __cplusplus
}
#endif

#endif /* LS1X_GMAC_HW_H_ */


