/*
 * Copyright (C) 2020-2021 Suzhou Tiancheng Software Ltd.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/*
 * LS1x_dma_hw.h
 *
 *  Created on: 2013-11-21
 *      Author: Bian
 */

#ifndef _LS1x_DMA_HW_H_
#define _LS1x_DMA_HW_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifndef bit
#define bit(x)	   (1<<x)
#endif

#define LOCK_DMA
#define UNLOCK_DMA

//-----------------------------------------------------------------------------

/*
 * DMA direction
 */
#define DMA_READ	0x01					/* DMA  */
#define DMA_WRITE	0x02					/* DMA д */

/*
 * DMA Control register Base Address
 */
#define LS1x_DMA_CTRL_ADDR		0xBFD01160
#define LS1x_DMA_CTRL			(*(volatile unsigned int*)(LS1x_DMA_CTRL_ADDR))

/*
 * DMA access address
 */
#define LS1x_DMA_ACCESS_ADDR	0xBFE78040	/* DMAдNAND flash(ID/STATUS)ʱķʵַ,
 	 	 	 	 	 	 	 	 	 	 	 * /дַͬ, дͨDMAʵ
 	 	 	 	 	 	 	 	 	 	 	 */

#define LS1x_AC97_TX_DMA_ADDR	0xBFE72420	/* AC97 DMA Ͳַ */
#define LS1x_AC97_RX_DMA_ADDR	0xBFE74C4C	/* AC97 DMA ղַ */

/*
 * DMA Control(ORDER_ADDR_IN) Register
 */
typedef enum
{
	dma_desc_addr_mask	= 0xFFFFFFC0,	// ѡDMAһַĸ26λ,6λΪ0;
	dma_desc_addr_shift	= 6,			// ʵַ?: ൱26λ Ask_addr6λ.
										/* һ:  64 ֽڶ ? (1<<6==64) */

	dma_stop			= bit(4),		// ûֹͣDMA; ɵǰݶд,ֹͣ.

	dma_start			= bit(3),		// ԿʼĵһDMA;
										// һصļĴغ,λ.
	ask_valid			= bit(2),		// û󽫵ǰDMAϢдصָڴַ;
										// ûдDMAϢ,λ.
	dma_dev_num_mask	= 0x03,			// bit 1:0, ѡDMA豸.
	dma_dev_nand		= 0x00,			// nand flash
	dma_dev_ac97_write	= 0x01,			// AC97 write device (codec)
	dma_dev_ac97_read	= 0x02,			// AC97 read device (codec)
} LS1x_dma_order_t;

/*
 * DMA Descriptor
 *
 * һ: ַΪ ORDER_ADDR_IN.ask_addr << 6, Ҫ 64 ֽڶ.
 *
 */
typedef struct LS1x_dma_desc
{
	unsigned int next_desc;				// 0x00	һַĴ - Ҫ 16 ֽڶ
	unsigned int mem_addr;				// 0x04	DMAڴַ
	unsigned int dev_addr;				// 0x08	豸ַĴ
	unsigned int length;				// 0x0C	ݳȼĴ - λ 4 ֽ
	unsigned int step_length;			// 0x10	ݴȼĴ
										//      ˵: ˵鱻ڴݿ֮ĳ,
										//      ǰһstepĽַһstepĿʼַ֮ļ.
	unsigned int step_times;			// 0x14	ݴѭĴ
										//      ˵: ѭ˵һDMAҪ˵ĿĿ;
										//      ֻһݿ,ѭĴֵԸֵΪ1.
	unsigned int command;				// 0x18	ƼĴ
	unsigned int _blank;                // 0x1C this is a gap of 64 bytes aligned
} LS1x_dma_desc_t;

/*
 * DMA Descriptor's ORDERADDR
 */
typedef enum
{
	next_desc_addr_mask  	= 0xFFFFFFFE,	// bit 31:1, 洢ڲһַĴ - Ҫ 16 ֽڶ
	next_desc_addr_shift 	= 1,
	next_desc_enable		= bit(0),		// һǷЧ, 1: Ч.
} LS1x_dma_desc2_t;

/*
 * DMA Descriptor's DADDR
 */
typedef enum
{
	desc_daddr_ac97_wren		 = bit(31),		// AC97дʹܣ1ʾд
	desc_daddr_ac97_stereo		 = bit(30),		// =0: mono; =1: 2 stereo
	desc_daddr_ac97_wrmode_mask	 = (0x03<<28),	// AC97дģʽ, =0: 1byte, =1: 2byte, =2: 4byte
	desc_daddr_ac97_wrmode_shift = 28,
	desc_daddr_ac97_wrmode_1	 = (0x00<<28),
	desc_daddr_ac97_wrmode_2	 = (0x01<<28),
	desc_daddr_ac97_wrmode_4 	 = (0x02<<28),

	desc_daddr_dev_mask		     = 0x0FFFFFFF,	// DMAAPB豸ַ

} LS1x_dma_daddr_t;

/*
 * DMA Descriptor's CMD
 */
typedef enum
{
	/* bit defination
	 */
	desc_cmd_addr_gen_mask	    = (0x03<<13),		// Դ/Ŀĵַɷʽ
	desc_cmd_addr_gen_shift	    = 13,
	desc_cmd_r_w			    = bit(12),			// DMA, =1: Ϊddr2д豸; =0Ϊ豸дddr2.

	/* write status
	 */
	desc_cmd_wr_status	   	    = (0x0F<<8),		// DMAд״̬
	desc_cmd_wr_shift           = 8,
	
	desc_cmd_wr_idle			= (0x00<<8),		// д״̬ڿ״̬
	desc_cmd_wr_ddr_wait		= (0x01<<8),		// DMAжҪִж豸дڴ,дڴ,
													// ڴ滹û׼Ӧ,DMAһֱڵȴڴӦ
	desc_cmd_wr_ddr			    = (0x02<<8),		// ڴDMAд,ǻûִд
	desc_cmd_wr_ddr_end		    = (0x03<<8),		// ڴDMAд,д;ʱDMAдڴ״̬
	desc_cmd_wr_dma_wait		= (0x04<<8),		// DMADMA״̬Ĵдڴ,ȴڴ
	desc_cmd_wr_dma			    = (0x05<<8),		// ڴдDMA״̬,ǲδ
	desc_cmd_wr_dma_end		    = (0x06<<8),		// ڴдDMA״̬
	desc_cmd_wr_step_end		= (0x07<<8),		// DMAһlengthȵĲ(Ҳ˵һstep)

	/* read status
	 */
	desc_cmd_rd_status	   	    = (0x0F<<4),		// DMA״̬
	desc_cmd_rd_shift           = 4,

	desc_cmd_rd_idle			 = (0x00<<4),		// ״̬ڿ״̬
	desc_cmd_rd_ready			 = (0x01<<4),		// յʼDMAstartźź,׼״̬,ʼ
	desc_cmd_rd_get_order		 = (0x02<<4),		// ڴ淢,ȴڴӦ
	desc_cmd_rd_order		     = (0x03<<4),		// ڴն,ִж
	desc_cmd_rd_finish_order_end = (0x04<<4),		// ڴDMA
	desc_cmd_rd_ddr_wait		 = (0x05<<4),		// DMAڴ淢,ȴڴӦ
	desc_cmd_rd_ddr			     = (0x06<<4),		// ڴDMA,ִжݲ
	desc_cmd_rd_ddr_end		     = (0x07<<4),		// ڴDMAһζ
	desc_cmd_rd_dev			     = (0x08<<4),		// DMA豸״̬
	desc_cmd_rd_dev_end		     = (0x09<<4),		// 豸ض,˴ζ豸
	desc_cmd_rd_step_end		 = (0x0A<<4),		// һstep, step times1
	
	desc_cmd_all_trans_over	    = bit(3),			// DMAִ걻õ
	desc_cmd_single_trans_over  = bit(2),			// DMAִһ
	desc_cmd_int			    = bit(1),			// DMAжź
	desc_cmd_int_enable         = bit(0),			// DMAжǷ
} LS1x_dma_cmd_t;

#ifdef __cplusplus
}
#endif

#endif /* LS1x_DMA_HW_H_ */

