/*
 * Copyright (C) 2020-2021 Suzhou Tiancheng Software Ltd.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/*
 * ls1x_pwm.h
 *
 * created: 2020/6/27
 *  author: Bian
 */

/****************************************************************************************
 * PWM ģʽ:
 *
 * 1. PULSE Ļߵεƽ.
 *
 * 2. TIMER ĻߵεĶʱ, ʱʱж.
 *          a. API װԶжӦʱж;
 *          b. API ʹûصӦʱж;
 *          c. API RTOSʹ event ׽ʱж.
 *
 *    TIMER жӦa/b/c֮һ, ȼ: a->b->c
 *
 *    TODO  ʱʱ, Щֵʹ!!!
 *
 * 3. оƬPWMſܱܸ, ʱPWM豸ڲʱ(źŲܴ).
 *
 ****************************************************************************************/

#ifndef _LS1X_PWM_H
#define _LS1X_PWM_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ls1x_io.h"

//-----------------------------------------------------------------------------

#ifdef BSP_USE_PWM0
extern void *devPWM0;
#endif
#ifdef BSP_USE_PWM1
extern void *devPWM1;
#endif
#ifdef BSP_USE_PWM2
extern void *devPWM2;
#endif
#ifdef BSP_USE_PWM3
extern void *devPWM3;
#endif

//-----------------------------------------------------------------------------

/*
 * PWM ģʽ
 */
#define PWM_SINGLE_PULSE    0x01    // single pulse
#define PWM_SINGLE_TIMER    0x02    // single timer, interval = hi_ns
#define PWM_CONTINUE_PULSE  0x04    // continue pulse
#define PWM_CONTINUE_TIMER  0x08    // continue timer, interval = hi_ns

/*
 * PWM Timer жϴص
 */

/*
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *          stopit  *stopit ֵ, öʱֹͣٹ, ʱԶ
 *                  hi_nsֵ, ȴһPWM﷧ֵж.
 */
typedef void (*pwmtimer_callback_t)(void *pwm, int *stopit);

/*
 * PWM open  arg 
 */
typedef struct pwm_cfg
{
    unsigned int hi_ns;             /* high level nano seconds,  ʱģʽ hi_ns */
    unsigned int lo_ns;             /* low  level nano seconds,  ʱģʽû lo_ns */
    int          mode;              /* pulse or timer, see above ʱģʽ */

    irq_handler_t       isr;        /* User defined interrupt handler */
    pwmtimer_callback_t cb;         /* called by interrupt handler */
#if BSP_USE_OS
    void               *event;      /* RTOS event created by user */
#endif
} pwm_cfg_t;

//-----------------------------------------------------------------------------
// driver operator
//-----------------------------------------------------------------------------

#if (PACK_DRV_OPS)

extern driver_ops_t *ls1x_pwm_drv_ops;

#define ls1x_pwm_init(pwm, arg)         ls1x_pwm_drv_ops->init_entry(pwm, arg)
#define ls1x_pwm_open(pwm, arg)         ls1x_pwm_drv_ops->open_entry(pwm, arg)
#define ls1x_pwm_close(pwm, arg)        ls1x_pwm_drv_ops->close_entry(pwm, arg)

#else

/*
 * PWMʼ
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *          arg       NULL
 *
 * :    0=ɹ
 */
int LS1x_PWM_initialize(void *dev, void *arg);

/*
 * PWM豸
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *          arg     : pwm_cfg_t *, PWM豸Ĺģʽ.
 *
 * :    0=ɹ
 *
 * ˵:    PWMģʽ, hi_nsǸߵƽ, lo_nsǵ͵ƽ.
 *          mode    PWM_SINGLE_PULSE:   
 *                  PWM_CONTINUE_PULSE: 
 *
 *          PWMڶʱģʽ, ʱʱʹhi_ns, (lo_ns). PWMʱPWM
 *          ʱж, ʱжӦ:
 *          1. ûԶж isr(!=NULL), Ӧisr;
 *          2. Զж isr=NULL, ʹPWMĬж, жϵcb صûʱӦ;
 *          3. Զж isr=NULLcb=NULL, event, PWMĬжϽPWM_TIMER_EVENT¼.
 *
 *          mode    PWM_SINGLE_TIMER:   ζʱ
 *                  PWM_CONTINUE_TIMER: ʱ
 *
 */
int LS1x_PWM_open(void *dev, void *arg);

/*
 * رPWMʱ
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *          arg     NULL
 *
 * :    0=ɹ
 */
int LS1x_PWM_close(void *dev, void *arg);

#define ls1x_pwm_init(pwm, arg)         LS1x_PWM_initialize(pwm, arg)
#define ls1x_pwm_open(pwm, arg)         LS1x_PWM_open(pwm, arg)
#define ls1x_pwm_close(pwm, arg)        LS1x_PWM_close(pwm, arg)

#endif

//-----------------------------------------------------------------------------
// user api
//-----------------------------------------------------------------------------

/*
 * PWM豸
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *          cfg     : pwm_cfg_t *, mode==PWM_SINGLE_PULSE/PWM_CONTINUE_PULSE
 *
 * :    0=ɹ
 */
int ls1x_pwm_pulse_start(void *pwm, pwm_cfg_t *cfg);
/*
 * ֹͣPWM豸
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *
 * :    0=ɹ
 */
int ls1x_pwm_pulse_stop(void *pwm);

/*
 * PWMʱ
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *          cfg     : pwm_cfg_t *, mode==PWM_SINGLE_TIMER/PWM_CONTINUE_TIMER
 *
 * :    0=ɹ
 */
int ls1x_pwm_timer_start(void *pwm, pwm_cfg_t *cfg);
/*
 * ֹͣPWMʱ
 * :    dev     devPWM0/devPWM1/devPWM2/devPWM3
 *
 * :    0=ɹ
 */
int ls1x_pwm_timer_stop(void *pwm);

#if defined(OS_RTTHREAD)
const char *ls1x_pwm_get_device_name(void *pwm);
#endif

#ifdef __cplusplus
}
#endif

#endif // _LS1X_PWM_H

