/*
 * ls1x_rtc.h
 *
 * created: 2020/6/30
 * authour: 
 */

/*
 * LS1B: ʹ
 *       1. д toywritehi/toywritelo, һֱ rtc_ctrl_ts λ. 
 *          ʹܺ: Զ, ȷؼʱ; toymatch ʱʹ
 *       2. д rtcwrite, һֱ rtc_ctrl_rs λ. 
 *          ʹܺ: rtcmatch ʹ
 *
 * LS1C: ʹ. Ϊ: д toytrim/rtctrim , ܷ.
 *
 */

#ifndef _LS1X_RTC_H
#define _LS1X_RTC_H

#ifdef __cplusplus
extern "C" {
#endif

//-----------------------------------------------------------------------------

#include <time.h>

#include "ls1x_io.h"

//-----------------------------------------------------------------------------

#define LS1X_RTC                0x0500
#define LS1X_TOY                0x0A00

/*
 * Virtual Sub-Device. ʱ豸
 */
#define DEVICE_RTCMATCH0       (LS1X_RTC | 1)
#define DEVICE_RTCMATCH1       (LS1X_RTC | 2)
#define DEVICE_RTCMATCH2       (LS1X_RTC | 3)

#define DEVICE_TOYMATCH0       (LS1X_TOY | 1)
#define DEVICE_TOYMATCH1       (LS1X_TOY | 2)
#define DEVICE_TOYMATCH2       (LS1X_TOY | 3)

//-----------------------------------------------------------------------------
// ʱжϻص
//-----------------------------------------------------------------------------

/*
 * :    device  RTC豸ж
 *          match   ǰRTCMATCHTOYMATCHļĴֵ
 *          stop    *stop ֵ, öʱֹͣٹ, ʱԶ
 *                  interval_msֵ, ȴһζʱʱֵж.
 */
typedef void (*rtctimer_callback_t)(int device, unsigned match, int *stop);

//-----------------------------------------------------------------------------
// RTC parameter
//-----------------------------------------------------------------------------

typedef struct rtc_cfg
{
    /*
     * This parameter's unit is millisecond(ms). After interrupt ocurred,
     * this value will auto loaded for next match interrupt.
     * if trig_datetime is NULL, toymatch use this parameter.
     *
     */
    int        interval_ms;

    /*
     * This parameter is used by toymatch. When toymatch arrives this datetime
     * toymatch interrupt will be triggered. This interrupt ocurred only once.
     *
     * if This parameter is NULL, use interval_ms as irq trigger interval
     *
     */
    struct tm *trig_datetime;

    irq_handler_t       isr;            /* User defined match-isr */
    rtctimer_callback_t cb;             /* called by match-isr */
#if BSP_USE_OS
    void               *event;          /* RTOS event created by user */
#endif
} rtc_cfg_t;

//-----------------------------------------------------------------------------
// ioctl command
//-----------------------------------------------------------------------------

#define IOCTL_SET_SYS_DATETIME      0x8001      // struct tm *
#define IOCTL_GET_SYS_DATETIME      0x8002      // struct tm *

/**
 * control rtc
 */
#define IOCTL_RTC_SET_TRIM          0x0811      // unsigned int *
#define IOCTL_RTC_GET_TRIM          0x0812      // unsigned int *

#define IOCTL_RTCMATCH_START        0x0813      // DEVICE_RTCMATCHx & rtc_cfg_t *
#define IOCTL_RTCMATCH_STOP         0x0814      // DEVICE_RTCMATCHx

/**
 * control toy
 */
#define IOCTL_TOY_SET_TRIM          0x0821      // unsigned int *
#define IOCTL_TOY_GET_TRIM          0x0822      // unsigned int *

#define IOCTL_TOYMATCH_START        0x0823      // DEVICE_TOYMATCHx & rtc_cfg_t *
#define IOCTL_TOYMATCH_STOP         0x0824      // DEVICE_TOYMATCHx

//-----------------------------------------------------------------------------
// RTC driver operators
//-----------------------------------------------------------------------------

#if (PACK_DRV_OPS)

extern driver_ops_t *ls1x_rtc_drv_ops;

#define ls1x_rtc_init(rtc, arg)             ls1x_rtc_drv_ops->init_entry(rtc, arg)
#define ls1x_rtc_open(rtc, arg)             ls1x_rtc_drv_ops->open_entry(rtc, arg)
#define ls1x_rtc_close(rtc, arg)            ls1x_rtc_drv_ops->close_entry(rtc, arg)
#define ls1x_rtc_read(rtc, buf, size, arg)  ls1x_rtc_drv_ops->read_entry(rtc, buf, size, arg)
#define ls1x_rtc_write(rtc, buf, size, arg) ls1x_rtc_drv_ops->write_entry(rtc, buf, size, arg)
#define ls1x_rtc_ioctl(rtc, cmd, arg)       ls1x_rtc_drv_ops->ioctl_entry(rtc, cmd, arg)

#else

/*
 * RTCʼ
 * :    dev      NULL
 *          arg     : struct tm *. ò NULL, ֵڳʼRTCϵͳʱ.
 *
 * :    0=ɹ
 */
int LS1x_RTC_initialize(void *dev, void *arg);

/*
 * RTCʱ
 * :    dev     Ҫ򿪵RTC豸 DEVICE_XXX
 *          arg     : rtc_cfg_t *, RTC豸Ĺģʽ
 *
 * :    0=ɹ
 *
 * ˵:    ʹõRTC豸, òrtc_cfg_tinterval_msֵ, RTCʱinterval_msֵʱ,
 *          RTCʱж, ʱжӦ:
 *          1. ûԶж isr(!=NULL), Ӧisr;
 *          2. Զж isr=NULL, ʹRTCĬж, жϵcb صûʱӦ;
 *          3. Զж isr=NULLcb=NULL, event, RTCĬжϽRTC_TIMER_EVENT¼.
 *
 *          ʹõTOY豸, interval_ms(>1000), ÷ʹRTC豸һ;
 *          interval_ms==0trig_datetime!=NULLʱ, ʾTOY豸ڼʱδʱʱж,
 *          жϴ̺һ.
 *          ʹtrig_datetimeжϽһ.
 *
 *          interval_msڼжϲһֱ; trig_datetimeڵʱжϽһ.
 *
 */
int LS1x_RTC_open(void *dev, void *arg);

/*
 * رRTCʱ
 * :    dev     ҪرյRTC豸 DEVICE_XXX
 *          arg     NULL
 *
 * :    0=ɹ
 */
int LS1x_RTC_close(void *dev, void *arg);

/*
 * ȡǰRTCʱ
 * :    dev     NULL
 *          buf     : struct tm *, ڴŶȡʱֵ
 *          size    : int, С=sizeof(struct tm)
 *          arg     NULL
 *
 * :    ȡֽ, Ϊsizeof(struct tm)
 */
int LS1x_RTC_read(void *dev, void *buf, int size, void *arg);

/*
 * RTCʱ
 * :    dev     NULL
 *          buf     : struct tm *, ڴŴдʱֵ
 *          size    : int, С=sizeof(struct tm)
 *          arg     NULL
 *
 * :    дֽ, Ϊsizeof(struct tm)
 */
int LS1x_RTC_write(void *dev, void *buf, int size, void *arg);

/*
 * RTCʱ豸
 * :    dev     NULL or DEVICE_XXX
 *
 *      ---------------------------------------------------------------------------------
 *          cmd                             |   arg
 *      ---------------------------------------------------------------------------------
 *          IOCTL_SET_SYS_DATETIME          |   : truct tm *
 *                                          |   ;: RTCϵͳʱֵ
 *      ---------------------------------------------------------------------------------
 *          IOCTL_GET_SYS_DATETIME          |   : struct tm *
 *                                          |   ;: ȡǰRTCϵͳʱֵ
 *      ---------------------------------------------------------------------------------
 *      ---------------------------------------------------------------------------------
 *          IOCTL_RTC_SET_TRIM              |   : unsigned int *
 *                                          |   ;: RTC32768HZʱƵֵ
 *      ---------------------------------------------------------------------------------
 *          IOCTL_RTC_GET_TRIM              |   : unsigned int *
 *                                          |   ;: ȡRTC32768HZʱƵֵ
 *      ---------------------------------------------------------------------------------
 *          IOCTL_RTCMATCH_START            |   : rtc_cfg_t *, RTCʱ
 *                                          |   dev==DEVICE_RTCMATCHx
 *      ---------------------------------------------------------------------------------
 *          IOCTL_RTCMATCH_STOP             |   : NULL, ֹͣRTCʱ
 *                                          |   dev==DEVICE_RTCMATCHx
 *      ---------------------------------------------------------------------------------
 *      ---------------------------------------------------------------------------------
 *          IOCTL_TOY_SET_TRIM              |   : unsigned int *
 *                                          |   ;: TOY32768HZʱƵֵ
 *      ---------------------------------------------------------------------------------
 *          IOCTL_TOY_GET_TRIM              |   : unsigned int *
 *                                          |   ;: ȡTOY32768HZʱƵֵ
 *      ---------------------------------------------------------------------------------
 *          IOCTL_TOYMATCH_START            |   : rtc_cfg_t *, TOYʱ
 *                                          |   dev==DEVICE_TOYMATCHx
 *      ---------------------------------------------------------------------------------
 *          IOCTL_TOYMATCH_STOP             |   : NULL, ֹͣTOYʱ
 *                                          |   dev==DEVICE_TOYMATCHx
 *      ---------------------------------------------------------------------------------
 *
 *
 * :    0=ɹ
 */
int LS1x_RTC_ioctl(void *dev, int cmd, void *arg);                 

#define ls1x_rtc_init(rtc, arg)             LS1x_RTC_initialize(rtc, arg)
#define ls1x_rtc_open(rtc, arg)             LS1x_RTC_open(rtc, arg)
#define ls1x_rtc_close(rtc, arg)            LS1x_RTC_close(rtc, arg)
#define ls1x_rtc_read(rtc, buf, size, arg)  LS1x_RTC_read(rtc, buf, size, arg)
#define ls1x_rtc_write(rtc, buf, size, arg) LS1x_RTC_write(rtc, buf, size, arg)
#define ls1x_rtc_ioctl(rtc, cmd, arg)       LS1x_RTC_ioctl(rtc, cmd, arg)

#endif

//-----------------------------------------------------------------------------
// User API
//-----------------------------------------------------------------------------

/*
 * RTCʱֵ, μls1x_rtc_read()
 */
int ls1x_rtc_set_datetime(struct tm *dt);
/*
 * ȡǰRTCʱ, μls1x_rtc_write()
 */
int ls1x_rtc_get_datetime(struct tm *dt);

/*
 * device is Virtual Sub Device
 */
/*
 * ʱ, μls1x_rtc_open()
 */
int ls1x_rtc_timer_start(unsigned device, rtc_cfg_t *cfg);
/*
 * رնʱ, μls1x_rtc_close()
 */
int ls1x_rtc_timer_stop(unsigned device);

/*
 * LS1x toymatch ڸʽת
 */
void ls1x_tm_to_toymatch(struct tm *dt, unsigned int *match);
void ls1x_toymatch_to_tm(struct tm *dt, unsigned int match);

unsigned int ls1x_seconds_to_toymatch(unsigned int seconds);
unsigned int ls1x_toymatch_to_seconds(unsigned int match);

/*
 * struct tm ڸʽת, +1900/-1900
 */
void normalize_tm(struct tm *tm, bool tm_format);

#ifdef __cplusplus
}
#endif

#endif // _LS1X_RTC_H

