/*
 * Copyright (C) 2020-2021 Suzhou Tiancheng Software Ltd.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/*
 * ls1x_spi_hw.h
 *
 *  Created on: 2013-11-1
 *      Author: Bian
 */

#ifndef LS1x_SPI_HW_H_
#define LS1x_SPI_HW_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifndef bit
#define bit(x)  (1<<x)
#endif

/*
 * SPI Flash fast access memory base
 */
#define FLASH_MEMBASE			0xBFC00000

/*
 * spi work at polling mode.
 */
#define LS1x_SPI_FASTREAD	0

/*
 * SPI Registers.
 */
typedef struct LS1x_SPI_regs
{
	volatile unsigned char ctrl;				// 0x00		ƼĴ
	volatile unsigned char sr;					// 0x01		״̬Ĵ
	union
	{
		volatile unsigned char txfifo;			// 0x02		ݽռĴ
		volatile unsigned char rxfifo;			// 0x02		ݷͼĴ
	} data;
	volatile unsigned char er;					// 0x03		ⲿĴ

	// SPI Flash
	volatile unsigned char param;				// 0x04		SPI Flash ƼĴ
	volatile unsigned char cs;					// 0x05		ƬѡƼĴ
	volatile unsigned char timing;				// 0x06		SPI Flash ʱƼĴ
} LS1x_SPI_regs_t;

typedef enum
{
	/* ƼĴ
	 */
	spi_ctrl_ien			= bit(7),	// SPCR(7)		жʹź, Ч
	spi_ctrl_en 			= bit(6),	// SPCR(6)		ϵͳʹź, Ч
	spi_ctrl_master 		= bit(4),	// SPCR(4)		masterģʽѡλ, λһֱ1
	spi_ctrl_cpol 			= bit(3),	// SPCR(3)		ʱӼλ
	spi_ctrl_cpha 			= bit(2),	// SPCR(2)		ʱλλ. =1: λ෴; =0: ͬ.
	spi_ctrl_spr_mask 		= 0x03,		// SPCR(1:0)	sclk_oƵ趨, Ҫsperspreһʹ
	spi_ctrl_spr_shift 		= 0,

	/* ״̬Ĵ
	 */
	spi_sr_iflag 			= bit(7),	// SPSR(7)		жϱ־λ, =1: ʾж, д1.
	spi_sr_txoverflow		= bit(6),	// SPSR(6)		дĴ־λ, Ϊ1ʾѾ,д1
	spi_sr_txfull 			= bit(3),	// SPSR(3)		дĴ־, 1ʾѾ
	spi_sr_txempty 			= bit(2),	// SPSR(2)		дĴձ־, 1ʾ
	spi_sr_rxfull 			= bit(1),	// SPSR(1)		Ĵ־, 1ʾѾ
	spi_sr_rxempty 			= bit(0),	// SPSR(0)		Ĵձ־, 1ʾ

	/* ⲿĴ
	 */
	spi_er_icnt_mask 		= 0xC0,		// SPER(7:6)	ڴٸֽںͳжź
	spi_er_icnt_shift 		= 6,
	spi_er_mode 			= bit(2),	// SPER(2)		spiӿģʽ, =0: 뷢ʱͬʱ; =1: 뷢ʱ.
	spi_er_spre_mask 		= 0x03,		// SPER(1:0)	sprһ趨Ƶı
	spi_er_spre_shift 		= 0,

	/* SPI FlashƼĴ
	 */
	spi_param_clk_div_mask	= 0xF0,		// SFC_PARAM(7:4)	ʱӷƵѡ, Ƶϵspre,sprͬ.
	spi_param_clk_div_shift = 4,
	spi_param_dual_io		= bit(3),	// SFC_PARAM(3)		ʹ˫I/Oģʽȼڿٶģʽ
	spi_param_fast_read		= bit(2),	// SFC_PARAM(2)		ʹÿٶģʽ
	spi_param_burst_en		= bit(1),	// SFC_PARAM(1)		spi flash ַ֧ģʽ
	spi_param_memory_en		= bit(0),	// SFC_PARAM(0)		spi flash ʹ, Чʱcsn[0].

	/* SPI ƬѡƼĴ
	 */
	spi_chip_sel_3			= bit(7),	// csnֵ
	spi_chip_sel_2			= bit(6),
	spi_chip_sel_1			= bit(5),
	spi_chip_sel_0			= bit(4),
	spi_chip_sel_3_en		= bit(3),	// Ϊ1ʱӦλcs7:4λ
	spi_chip_sel_2_en		= bit(2),
	spi_chip_sel_1_en		= bit(1),
	spi_chip_sel_0_en		= bit(0),

	/* SPI FlashʱƼĴ
	 */
	spi_timing_tcsh_mask	= 0x03,		// SPI Flash ƬѡźЧʱ䣬ԷƵʱT
	spi_timing_tcsh_shift	= 0,
	spi_timing_tcsh_1		= 0x00,
	spi_timing_tcsh_2		= 0x01,
	spi_timing_tcsh_4		= 0x02,
	spi_timing_tcsh_8		= 0x03,
} SPI_controller_t;

/*
 * Ƶϵֵ
 */
typedef struct LS1x_SPI_clkdiv
{
	unsigned char	spre;
	unsigned char	spr;
	unsigned int    sprate;
} spi_clkdiv_t;

#ifdef __cplusplus
}
#endif

#endif /* LS1x_SPI_HW_H_ */

